import { PublicKey, SystemProgram, SYSVAR_RENT_PUBKEY, } from '@solana/web3.js';
import { getTokenBridgeDerivedAccounts, } from '@certusone/wormhole-sdk/lib/cjs/solana';
import { deriveClaimKey, derivePostedVaaKey, } from '@certusone/wormhole-sdk/lib/cjs/solana/wormhole';
import { ASSOCIATED_TOKEN_PROGRAM_ID, getAssociatedTokenAddressSync, TOKEN_PROGRAM_ID, } from '@solana/spl-token';
import { isBytes, parseTokenTransferVaa, } from '@certusone/wormhole-sdk';
import { deriveEndpointKey, deriveMintAuthorityKey, deriveRedeemerAccountKey, deriveTokenBridgeConfigKey, deriveWrappedMetaKey, deriveWrappedMintKey, } from '@certusone/wormhole-sdk/lib/cjs/solana/tokenBridge';
import { aggUserUpgradePointsKey, bridgeKey, foreignContractKey, redeemerConfigKey, totalUpgradePointsKey, transferOutOfBandV0Key, transferV2Key, upgradePointsKey, upgradeRewardsEscrow, } from './pdas';
import BN from 'bn.js';
import { toBN } from '@render-foundation/spl-utils';
import { emissionScheduleReceiptKey, } from '@render-foundation/emission-distributor-sdk';
import { mintWindowedBreakerKey } from '@render-foundation/circuit-breaker-sdk';
export function receiveRenderOutOfBoundV0Ix(program, payer, tokenBridgeProgramId, wormholeProgramId, wormholeMessage, accts) {
    const parsed = isBytes(wormholeMessage)
        ? parseTokenTransferVaa(wormholeMessage)
        : wormholeMessage;
    const wrappedMint = deriveWrappedMintKey(tokenBridgeProgramId, parsed.tokenChain, parsed.tokenAddress);
    const tokenBridgeAccounts = getCompleteTransferWrappedWithPayloadCpiAccounts(tokenBridgeProgramId, wormholeProgramId, payer, parsed);
    const recipient = new PublicKey(parsed.tokenTransferPayload.subarray(2, 34));
    console.log(`recipientbar ${recipient}`);
    return program.methods
        .receiveRenderOutOfBandV0({ vaaHash: [...parsed.hash] })
        .accounts({
        config: redeemerConfigKey(program.programId)[0],
        foreignContract: foreignContractKey(parsed.emitterChain)[0],
        recipient,
        transfer: transferOutOfBandV0Key(parsed.hash)[0],
        rndrMint: accts.rndrMint,
        circuitBreaker: accts.circuitBreakerK,
        bridge: accts.bridgeK,
        payerTokenAccount: getAssociatedTokenAddressSync(wrappedMint, new PublicKey(payer)),
        tokenBridgeProgram: new PublicKey(tokenBridgeProgramId),
        wrndrMint: tokenBridgeAccounts.tokenBridgeWrappedMint,
        ...tokenBridgeAccounts,
    })
        .instruction();
}
export function createReceiveRndrV2Ix(program, payer, tokenBridgeProgramId, wormholeProgramId, wormholeMessage, accts) {
    const parsed = isBytes(wormholeMessage)
        ? parseTokenTransferVaa(wormholeMessage)
        : wormholeMessage;
    const wrappedMint = deriveWrappedMintKey(tokenBridgeProgramId, parsed.tokenChain, parsed.tokenAddress);
    const tokenBridgeAccounts = getCompleteTransferWrappedWithPayloadCpiAccounts(tokenBridgeProgramId, wormholeProgramId, payer, parsed);
    const recipient = new PublicKey(parsed.tokenTransferPayload.subarray(1, 33));
    /*const recipientTokenAccount = getAssociatedTokenAddressSync(
      accts.rndrMint,
      recipient
    )*/
    return program.methods
        .receiveRenderV2({ vaaHash: [...parsed.hash] })
        .accounts({
        config: redeemerConfigKey(program.programId)[0],
        foreignContract: foreignContractKey(parsed.emitterChain)[0],
        recipient,
        transfer: transferV2Key(parsed.hash)[0],
        rndrMint: accts.rndrMint,
        circuitBreaker: accts.circuitBreakerK,
        bridge: accts.bridgeK,
        payerTokenAccount: getAssociatedTokenAddressSync(wrappedMint, new PublicKey(payer)),
        tokenBridgeProgram: new PublicKey(tokenBridgeProgramId),
        wrndrMint: tokenBridgeAccounts.tokenBridgeWrappedMint,
        ...tokenBridgeAccounts,
    })
        .instruction();
}
// Temporary
export function getCompleteTransferWrappedWithPayloadCpiAccounts(tokenBridgeProgramId, wormholeProgramId, payer, vaa) {
    const parsed = isBytes(vaa) ? parseTokenTransferVaa(vaa) : vaa;
    const mint = deriveWrappedMintKey(tokenBridgeProgramId, parsed.tokenChain, parsed.tokenAddress);
    const cpiProgramId = new PublicKey(parsed.to);
    return {
        payer: new PublicKey(payer),
        tokenBridgeConfig: deriveTokenBridgeConfigKey(tokenBridgeProgramId),
        vaa: derivePostedVaaKey(wormholeProgramId, parsed.hash),
        tokenBridgeClaim: deriveClaimKey(tokenBridgeProgramId, parsed.emitterAddress, parsed.emitterChain, parsed.sequence),
        tokenBridgeForeignEndpoint: deriveEndpointKey(tokenBridgeProgramId, parsed.emitterChain, parsed.emitterAddress),
        tokenBridgeRedeemer: deriveRedeemerAccountKey(cpiProgramId),
        tokenBridgeWrappedMint: mint,
        tokenBridgeWrappedMeta: deriveWrappedMetaKey(tokenBridgeProgramId, mint),
        tokenBridgeMintAuthority: deriveMintAuthorityKey(tokenBridgeProgramId),
        rent: SYSVAR_RENT_PUBKEY,
        systemProgram: SystemProgram.programId,
        tokenProgram: TOKEN_PROGRAM_ID,
        wormholeProgram: new PublicKey(wormholeProgramId),
    };
}
export const initializeUpgradeRewardsV0Ix = (program, rndrMint, args, accts) => {
    return program.methods
        .initializeUpgradeRewardsV0({
        authority: args.authority,
        backfillAuthority: args.backfillAuthority ?? args.authority,
        upgradesEscrowConfig: args.upgradesEscrowConfig,
    })
        .accounts({
        rndrMint: rndrMint,
        rndrMintBreaker: mintWindowedBreakerKey(rndrMint)[0],
        upgradeRewardsEscrow: upgradeRewardsEscrow(rndrMint)[0],
        authority: accts.authority,
    })
        .instruction();
};
export const distributeUpgradeRewardsV1Ix = (program, epochId, rndrMint, accts) => {
    const { recipient, payer } = accts;
    return program.methods
        .distributeUpgradeRewardsV1({
        epochId: new BN(epochId),
    })
        .accounts({
        recipient,
        upgradePoints: upgradePointsKey(recipient, epochId)[0],
        totalPoints: totalUpgradePointsKey(epochId)[0],
        aggUserUpgradePoints: aggUserUpgradePointsKey(recipient)[0],
        rndrMint,
        //emissionSchedule: emissionScheduleKey(rndrMint.address)[0],
        emissionScheduleReceipt: emissionScheduleReceiptKey(rndrMint, epochId)[0],
        associatedTokenProgram: ASSOCIATED_TOKEN_PROGRAM_ID,
        payer,
    })
        .instruction();
};
export function createInitializeIx(program, tokenBridgeProgramId, wormholeProgramId, accts, args) {
    const { owner, wRndrMint, rndrMint, bridgeK, circuitBreakerK } = accts;
    const tokenBridgeAccounts = getTokenBridgeDerivedAccounts(program.programId, tokenBridgeProgramId, wormholeProgramId);
    return program.methods
        .initializeBridgeV1({
        ...args,
        incentiveThreshold: toBN(args.incentiveThreshold, rndrMint.decimals),
    })
        .accounts({
        owner: owner,
        bridge: bridgeKey(rndrMint.address, wRndrMint, program.programId)[0],
        rndrMint: rndrMint.address,
        wrndrMint: wRndrMint,
        circuitBreaker: circuitBreakerK,
        redeemerConfig: redeemerConfigKey(program.programId)[0],
        tokenBridgeProgram: new PublicKey(tokenBridgeProgramId),
        wormholeProgram: new PublicKey(wormholeProgramId),
        ...tokenBridgeAccounts,
    })
        .instruction();
}
export async function createRegisterForeignContractIx(program, payer, tokenBridgeProgramId, chain, contractAddress, tokenBridgeForeignAddress, rndrMint, wrndrMint) {
    return program.methods
        .registerForeignContractV1({ chain, address: [...contractAddress] })
        .accounts({
        owner: new PublicKey(payer),
        rndrMint,
        bridge: bridgeKey(rndrMint, wrndrMint)[0],
        wrndrMint,
        foreignContract: foreignContractKey(chain)[0],
        tokenBridgeForeignEndpoint: deriveEndpointKey(tokenBridgeProgramId, chain, tokenBridgeForeignAddress),
        tokenBridgeProgram: new PublicKey(tokenBridgeProgramId),
    })
        .instruction();
}
//# sourceMappingURL=instructions.js.map